%{
@Name:Ai_continuous.m
@Author: Smacq
@Copyright: Smacq
@Date: 29-11-2024
@Brief: 
The Ai functions of 1252A_DAQ, continuous acquisition.
Acquisition Ai0 channel 100 data point of 10 times.
You can loot out user manual know more about functions or Device info.

1252A_DAQ only connect USB execution result:
ans =
     0
ans =
     0
ans =
     0
ans =
     0
ans =
     0
ans =
     0
ans =
     0
ans =
     0
remain_buffer =
     1999992
Ai_data_array(1:100):
ans =
  1100 single 
 1  10
    0.0032    0.0032    0.0032    0.0032    0.0032    0.0032    0.0032    0.0032    0.0032    0.0032
...
   91  100
    0.0032    0.0032    0.0032    0.0032    0.0032    0.0032    0.0032    0.0032    0.0032    0.0005
ans =
     0
ans =
     0
ans =
     0

Error code:
Execution successful  =  0
Device not found      = -1
DevIndex overflow     = -2
Device firmware error = -3
Device closed         = -4
Transfer data error   = -5
PC Out of memory      = -6
Execution timeout     = -7
Reading thread not on = -8

mex -setup
MEX setup 'Microsoft Visual C++ 2022 (C)' to compile in C++.

To choose a different language, please select a command from the following options:
 mex -setup C++ 
 mex -setup FORTRAN

MEX setup 'Microsoft Visual C++ 2022' to compile in C++.
%}

clear
clc
% Check MATLAB bit is 32 or 64, load diffrent lib file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\usb-1000.dll';  
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\usb-1000.dll';  
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\usb-1000.h';

try
    loadlibrary(dll_file_path,h_file_path);
    if libisloaded('usb0x2D1000')
        fprintf('Dll load success.\n')
        %libfunctions usb0x2D1000 -full
    end
catch
    fprintf('Dll load Fail pleaes check matlab bit and lib file path.\n')
end

Ai_data_array = zeros(1,1600);
%{
Init Ai_data_array:
Usually the array size need >= acquisition data points size.
Data type is float32, it is not modifiable.
The data points saved to Ai_data_array arrange from opened channel Ai0~15
%}

%Define pointer to Ai_data_array type is singlePtr
Ai_data_array_ptr = libpointer('singlePtr',Ai_data_array);

%Device index starts from zero
Dev_Index = 0;

Range = 5;
%{
Ai voltage range:
5 = -5~5V or 10 = 0~10V
1252A voltage range param only support  5/10
%}

Mode = 1;
%{
Ai channel mode:
0=diff or 1=NRSE or 3=RSE
1252A only support param 0/1/3
%}

%Software tigger turn on
TrigOpen = 1;
%Software tigger turn off
TrigClose = 0;

Num = 100;
%{
Acquisition amount of data:
Each opened Ai channel will get this amount of data.
Suggest setting this value within sampling rate 10%~20%.
%}

Channel = 1;
%{
Configuration values for Ai channels:
1=0x0001=16-bit(0000 0000 0000 0001) Ai15~1 is CLOSE and Ai0 is OPEN.
3=0x0003=16-bit(0000 0000 0000 0011) Ai15~2 is CLOSE and Ai1~0 is OPEN.
65535=0xFFFF=16-bit(1111 1111 1111 1111) Ai15~0 is OPEN.
%}

Samplerate = 1000;
%{
Ai Sampling rate unit Hz:
1252A is asynchronous DAQ, single channel max rate is 500000.
Multichannel max rate is 200000 and each channel share out max sample rate.
%}

Timeout = 200;
%{
Timeout unit ms:
Suggest Timeout>(Num*Channel_count/Samplerate)*1000
Avoid GetAiChans() func timeout, if timeout return -7.
%}

%Open Device, return value assignment and print error_code.
calllib('usb0x2D1000','OpenDevice',Dev_Index)
%Reset and init Device
calllib('usb0x2D1000','ResetDevice',Dev_Index)
%Set Device Ai input channel mode
calllib('usb0x2D1000','SetUSB1AiRange',Dev_Index,Range)
%Set Device Ai input voltage range
calllib('usb0x2D1000','SetSampleRate',Dev_Index,Samplerate)
%Set Device sample rate
calllib('usb0x2D1000','SetChanMode',Dev_Index,Mode)
%Set Device Ai channel open
calllib('usb0x2D1000','SetChanSel',Dev_Index,Channel)
%Turn on Device reading thread
calllib('usb0x2D1000','StartRead',Dev_Index)
%Turn on Device software trigger
calllib('usb0x2D1000','SetSoftTrig',Dev_Index,TrigOpen)

%Continuous acquisition execution 10 times GetAiChans()
for i = 1:10
    %{
    Ai_FIFO_buffer range 0~200000
    GetAiChans() return the remaining Device FIFO buffer value.
    FIFO buffer 100 data points start save to Ai_data_array_ptr is Ai_data_array(1:100)
    %}
    remain_buffer = calllib('usb0x2D1000','GetAiChans',Dev_Index,Num,Channel,Ai_data_array_ptr,Timeout)
    Ai_data_array = Ai_data_array_ptr.value;
    disp("Ai_data_array(1:100):")
    Ai_data_array(1:100)
    %Add your code here handle data
end

%Turn off Device reading thread
calllib('usb0x2D1000','StopRead',Dev_Index)
%Turn off Device software trigger
calllib('usb0x2D1000','SetSoftTrig',Dev_Index,TrigClose)
%Clear Device FIFO buffer
calllib('usb0x2D1000','ClearBufs',Dev_Index)
%Close Device
calllib('usb0x2D1000','CloseDevice',Dev_Index)

unloadlibrary('usb0x2D1000')
