%{
@Name: Counter.py
@Author: Smacq
@Copyright: Smacq
@Date: 2-12-2024
@Brief: The Counter functions of 1252A_DAQ, get 10s Ct0~3 counter value and print it.
You can loot out user manual know more about functions or Device info.

1252A_DAQ only connect USB execution result:
ans =
0
ans =
0
ans =
0
ans =
0
ans =
0
ans =
0
1 second:
ct_channel:0	Ct_count1
ct_channel:1	Ct_count1
ct_channel:2	Ct_count1
ct_channel:3	Ct_count1
...
10 second:
ct_channel:0	Ct_count1
ct_channel:1	Ct_count1
ct_channel:2	Ct_count1
ct_channel:3	Ct_count1
ans =
0
ans =
0
ans =
0

Error code:
Execution successful  =  0
Device not found      = -1
DevIndex overflow     = -2
Device firmware error = -3
Device closed         = -4
Transfer data error   = -5
PC Out of memory      = -6
Execution timeout     = -7
Reading thread not on = -8

mex -setup
MEX setup 'Microsoft Visual C++ 2022 (C)' to compile in C++.

To choose a different language, please select a command from the following options:
 mex -setup C++ 
 mex -setup FORTRAN

MEX setup 'Microsoft Visual C++ 2022' to compile in C++.
%}

clear
clc
% Check MATLAB bit is 32 or 64, load diffrent lib file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\usb-1000.dll';  
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\usb-1000.dll';  
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\usb-1000.h';
% load dll and .h file
try
    loadlibrary(dll_file_path,h_file_path);
    if libisloaded('usb0x2D1000')
        fprintf('Dll load success.\n')
        %libfunctions usb0x2D1000 -full
    end
catch
    fprintf('Dll load Fail pleaes check matlab bit and lib file path.\n')
end

%Device index starts from zero
Dev_Index = 0;
%Digital IO Transfer turn on
TransDioSwitch_On = 1;
%Digital IO Transfer turn off
TransDioSwitch_Off = 0;

%Configuration Counter ON
Counter_On = 1;
%Configuration Counter OFF
Counter_Off = 0;

CtrNum = 15;
%{
Configuration Counter channel:
1=0x1=4-bit(0001) Ct3~1 is CLOSE and Ct0 is OPEN.
15=0xF=4-bit(1111) Ct3~0 is OPEN.
%}

CtrMode = 0;
%{
Configuration Counter reading mode:
0=edge or 1=cycle or 2=positive pulse width or 3=negative pulse width
1252A only support param 0~3
%}

CtrEdge = 1;
%{
Set Counter edge level:
1=rising or 2=falling 
1252A only support param 1/2 
%}

%Open Device, return value assignment and print error_code.
calllib('usb0x2D1000','OpenDevice',Dev_Index)
%Reset and init Device
calllib('usb0x2D1000','ResetDevice',Dev_Index)
%Turn on Device reading thread
calllib('usb0x2D1000','StartRead',Dev_Index)
%Turn on Device Digital IO Transfer thread
calllib('usb0x2D1000','TransDioIn',Dev_Index,TransDioSwitch_On)
%Set Device Counter: Device_0, CrtNum_Ct0-3, CrtMode_1(cycle)
calllib('usb0x2D1000','SetCounter',Dev_Index,CtrNum,CtrMode,CtrEdge)
%Start Device Counter record
calllib('usb0x2D1000','StartCounter',Dev_Index,CtrNum,Counter_On)

% Run 10s for catch Ct0~3 signal, each second update input signal value and print
for second = 1:10
    pause(1); % Wait for 1 second
    fprintf('%d second:\n', second);
    for ct_channel = 0:3
        % GetCounter() only support CrtMode=0(edge), return edge count
        if CtrMode == 0
            ct_value = calllib('usb0x2D1000','GetCounter',Dev_Index,ct_channel);
            fprintf('ct_channel:%d\tCt_count%d\n', ct_channel, ct_value);
        else
            % GetCtrTime() not support CrtMode=0(edge), return value is time unit us
            ct_time = calllib('usb0x2D1000','GetCtrTime',Dev_Index,ct_channel);
            fprintf('ct_channel:%d\tCt_time(us)%f\n', ct_channel, ct_time);
        end
    end
end

%Turn off Device Counter Ct0~3
calllib('usb0x2D1000','StartCounter',Dev_Index,CtrNum,Counter_Off)
%Turn off Device Digital IO transfer thread
calllib('usb0x2D1000','TransDioIn',Dev_Index,TransDioSwitch_Off)
%Turn off Device reading thread
calllib('usb0x2D1000','StopRead',Dev_Index)
%Close Device
calllib('usb0x2D1000','CloseDevice',Dev_Index)

unloadlibrary('usb0x2D1000')