%{
@Name:Digital_io.m
@Author: Smacq
@Copyright: Smacq
@Date: 29-11-2024
@Brief: The digital I/O functions of 1252A_DAQ, set Do output HIGH/LOW and read Di input value.
Digital voltage range:
Do Pin HIGH is 3.3V and LOW is 0V, Di Pin HIGH is 3.3-5V and LOW is 0-1.2V.
You can loot out user manual know more about functions or Device info.

1252A_DAQ only connect USB execution result:
ans =
     0
ans =
     0
ans =
     0
ans =
     0
error_code =
     0
Now Do15-0 is all HIGH
error_code =
     0
Now Do15-0 is all LOW

DioOut =
     1
error_code =
     0
DO_bit =
     0
is HIGH, other DO is LOW.
...
DioOut =
       32768
error_code =
     0
DO_bit =
    15
is HIGH, other DO is LOW.

Di_value =
       65535
...
Di_value =
       65535
ans =
     0
ans =
     0

Error code:
Execution successful  =  0
Device not found      = -1
DevIndex overflow     = -2
Device firmware error = -3
Device closed         = -4
Transfer data error   = -5
PC Out of memory      = -6
Execution timeout     = -7
Reading thread not on = -8

mex -setup
MEX setup 'Microsoft Visual C++ 2022 (C)' to compile in C++.

To choose a different language, please select a command from the following options:
 mex -setup C++ 
 mex -setup FORTRAN

MEX setup 'Microsoft Visual C++ 2022' to compile in C++.
%}

clear
clc
% Check MATLAB bit is 32 or 64, load diffrent lib file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\usb-1000.dll';  
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\usb-1000.dll';  
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\usb-1000.h';
% load dll and .h file
try
    loadlibrary(dll_file_path,h_file_path);
    if libisloaded('usb0x2D1000')
        fprintf('Dll load success.\n')
        %libfunctions usb0x2D1000 -full
    end
catch
    fprintf('Dll load Fail pleaes check matlab bit and lib file path.\n')
end

%Device index starts from zero
Dev_Index = 0;
%Digital IO Transfer turn on
TransDioSwitch_On = 1;
%Digital IO Transfer turn off
TransDioSwitch_Off = 0;

%Open Device, return value assignment and print error_code.
calllib('usb0x2D1000','OpenDevice',Dev_Index)
%Reset and init Device
calllib('usb0x2D1000','ResetDevice',Dev_Index)
%Turn on Device reading thread
calllib('usb0x2D1000','StartRead',Dev_Index)
%Turn on Device Digital IO Transfer thread
calllib('usb0x2D1000','TransDioIn',Dev_Index,TransDioSwitch_On)

%Set All Do Pin output HIGH and then LOW, interval of 1 second.
for i =1:3
    %Set Do Pin output, 65535=0xFFFF=16-bit(1111 1111 1111 1111) Do15-0 all HIGH.
    error_code = calllib('usb0x2D1000','SetDioOut',Dev_Index,65535)
    if error_code == 0
        fprintf("Now Do15-0 is all HIGH")
    end
    %Necessary delay for easy observation
    pause(1)
    %Set Do Pin output, 0=0x0000=16-bit(0000 0000 0000 0000) Do15-0 all LOW.
    error_code = calllib('usb0x2D1000','SetDioOut',Dev_Index,0)
    if error_code == 0
        fprintf("Now Do15-0 is all LOW")
    end
    %Necessary delay for easy observation
    pause(1)
end

%Set single DO_Pin output HIGH, from Do0 to Do15 other Do PIN is LOW.
for DO_bit = 0:15
    %1=0x0001=16-bit(0000 0000 0000 0001) Do15-1 is LOW and Do0 is HIGH, and the like.
    DioOut = bitshift(1,DO_bit)
    error_code = calllib('usb0x2D1000','SetDioOut',Dev_Index,DioOut)
    if error_code == 0
        DO_bit
        fprintf("is HIGH, other DO is LOW.")
    end
    pause(1)
end

%Get Di Pin input value in 10s, update time is 1s (when Di Pin is FLOATING default HIGH).
for second = 1:10
    %Di_value is 65535=0xFFFF=16-bit(1111 1111 1111 1111) Di15-0 input Pin value all HIGH.
    Di_value = calllib('usb0x2D1000','GetDioIn',Dev_Index)
    pause(1)
end

%Turn off Device Digital IO transfer thread
calllib('usb0x2D1000','TransDioIn',Dev_Index,TransDioSwitch_Off)
%Turn off Device reading thread
calllib('usb0x2D1000','StopRead',Dev_Index)
%Close Device
calllib('usb0x2D1000','CloseDevice',Dev_Index)

unloadlibrary('usb0x2D1000')
